%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Hybrid Mobility Model                                                %
% Author: Wenbo Wang                                                   %
% Date: 1st Aug. 2015 -- the 1st version                               %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% Note :1.in this model, the normal distribution is used with confident% 
%       level 95% to model velocity and angles.                        %
%       2. in this model, together with the author's thesis,  %
%       we can see in the x-y plane, a user moves inside a small area  %
%       with high probability. This is used to simulate office scenario%
%       and some other indoor scenarios. In this model, this process is%
%       simplified, in every floor the user is assumed to move around  %
%       the centre. If you want to change the scenario (moving around  %
%       the centre), the variable such like 'dist' is needed to modify %
%       and the process part is needed to modify.                      %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all the figures, clear all the variables in the workspace and clear
% the screen
close all;
clear all;
clc;

%% variable
xmax=50;ymax=50;        % define x-y plane

noOfFloor=3;            % the number of floors
floorHeight=3;          % the height spacing of floors

n=10000;
n=2*ceil(n/2);          % simulation times (pause also counted as 1 time simulation)

v_min=0.5;v_max=3;      % define the upper and lower bound of velocity [m/s]
%v_min=0.89;v_max=1.083;% this is used to simulate elder people

r_min=0;r_max=2*pi;     % define the upper and lower bound of angle [radian]

t_move_min=3;t_move_max=10;% define the move time bound [s]

t_pause_min=0.5;t_pause_max=30;% define the pause time bound [s]

dist_min=0.5;dist_max=20;% define the upper and lower bound of distance away from boundry [m]

%% avariable models
v=setVariable(v_min,v_max,'normal',n/2); % form velocity set
r=setVariable(r_min,r_max,'uniform',n/2);% form angle set
t_m=setVariable(t_move_min,t_move_max,'normal',n/2);% form the moving time set
t_p=setVariable(t_pause_min,t_pause_max,'normal',n/2);% form the pause time set
dist=setVariable(dist_min,dist_max,'uniform',n);% form the shortest distance between the user and the nearest boundary

% define the total simulation time [s]
% the first row is move time, second row is pause time
t(1,:)=t_m;
t(2,:)=t_p;

% define the sampling frequency [Hz]
Fs=1e2;

% define the starting point x-y-z
temp_3=0:floorHeight:floorHeight*(noOfFloor-1);
%startP=[0,0,0];
startP=[xmax*rand,ymax*rand,temp_3(randi(length(temp_3)))];

% you can predefine the starting point and load it
%load('para3.mat');

%% process

% initial

% these two lines are alter the float time form into an integer one
t=floor(t*Fs);
t=[zeros(2,1),t];

% the starting point
trace(1:3,1)=startP;

counta=1;% for angle purpose, counter
countb=1;% for dist purpose, counter

% processing...
for p=1:n/2
    
    % 'count1' and 'count2' are moving/pause duration
    count1=sum(t(1,1:p))+1;
    count2=sum(t(1,1:p+1));
    
    % angle
    ang=r(counta);
    
    for pp=count1:count2
        
        % computing the coordinate of points in 3-D
        trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
        trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        temp_3=thirdDi(1,1/Fs,trace(3,pp),0,(noOfFloor-1)*floorHeight);
        trace(3,pp+1)=temp_3(2);
        
        temp1=trace(1,pp+1)+dist(countb);
        temp2=trace(2,pp+1)+dist(countb);
        temp3=trace(1,pp+1)-dist(countb);
        temp4=trace(2,pp+1)-dist(countb);
        
        % the following 'if's use the same algorithm as the random
        % direction mobility model
        if temp1<xmax&&temp3>0&&temp2<ymax&&temp4>0
            continue;
        elseif temp1>xmax&&temp4<0
            ang=setVariable(pi/2,pi,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        elseif temp1>xmax&&temp2>ymax
            ang=setVariable(pi,3*pi/2,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        elseif temp3<0&&temp2>ymax
            ang=setVariable(3*pi/2,2*pi,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        elseif temp3<0&&temp4<0
            ang=setVariable(0,pi/2,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        elseif temp1>=xmax&&temp4>=0&&temp2<=ymax
            ang=setVariable(pi/2,3*pi/2,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        elseif temp3<=0&&temp4>=0&&temp2<=ymax
            ang=setVariable(3*pi/2,5*pi/2,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        elseif temp2>=ymax&&temp3>=0&&temp1<=xmax
            ang=setVariable(pi,2*pi,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        elseif temp4<=0&&temp3>=0&&temp1<=xmax
            ang=setVariable(0,pi,'normal',1);
            trace(1,pp+1)=trace(1,pp)+v(p)*cos(ang)/Fs;
            trace(2,pp+1)=trace(2,pp)+v(p)*sin(ang)/Fs;
        end
        
    end
    
    counta=counta+1;
    countb=countb+1;
    
end

tm=floor(t_m*Fs);
tp=floor(t_p*Fs);

% the following process is to combine the pause time part with moving time
% part
for ppp=1:n/2
    
    trace_rss(1,1+sum(tm(1:ppp-1))+sum(tp(1:ppp-1)):1+sum(tm(1:ppp))+sum(tp(1:ppp-1)))=trace(1,1+sum(tm(1:ppp-1)):1+sum(tm(1:ppp)));
    trace_rss(1,2+sum(tm(1:ppp))+sum(tp(1:ppp))-tp(ppp):1+sum(tm(1:ppp))+sum(tp(1:ppp)))=trace(1,1+sum(tm(1:ppp)));
    trace_rss(2,1+sum(tm(1:ppp-1))+sum(tp(1:ppp-1)):1+sum(tm(1:ppp))+sum(tp(1:ppp-1)))=trace(2,1+sum(tm(1:ppp-1)):1+sum(tm(1:ppp)));
    trace_rss(2,2+sum(tm(1:ppp))+sum(tp(1:ppp))-tp(ppp):1+sum(tm(1:ppp))+sum(tp(1:ppp)))=trace(2,1+sum(tm(1:ppp)));
    trace_rss(3,1+sum(tm(1:ppp-1))+sum(tp(1:ppp-1)):1+sum(tm(1:ppp))+sum(tp(1:ppp-1)))=trace(3,1+sum(tm(1:ppp-1)):1+sum(tm(1:ppp)));
    trace_rss(3,2+sum(tm(1:ppp))+sum(tp(1:ppp))-tp(ppp):1+sum(tm(1:ppp))+sum(tp(1:ppp)))=trace(3,1+sum(tm(1:ppp)));
    
end

% save the data
save('hybrid_model_2_35.mat');

